@@ -1,4 +1,4 @@
-#! /usr/bin/perl -w
+#! /usr/bin/perl
 
 # debchange: update the debian changelog using your favorite visual editor
 # For options, see the usage message below.
@@ -13,7 +13,7 @@
 # Originally by Christoph Lameter <clameter@debian.org>
 # Modified extensively by Julian Gilbey <jdg@debian.org>
 #
-# Copyright 1999-2005 by Julian Gilbey 
+# Copyright 1999-2005 by Julian Gilbey
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,19 +26,20 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 use 5.008;  # We're using PerlIO layers
 use strict;
+use warnings;
 use open ':utf8';  # changelogs are written with UTF-8 encoding
 use filetest 'access';  # use access rather than stat for -w
 # for checking whether user names are valid and making format() behave
 use Encode qw/decode_utf8 encode_utf8/;
-use Getopt::Long;
+use Getopt::Long qw(:config gnu_getopt);
 use File::Copy;
 use File::Basename;
 use Cwd;
+use Dpkg::Compression;
 use lib '/usr/share/devscripts';
 use Devscripts::Debbugs;
 
@@ -94,7 +95,7 @@ Options:
          editor opened.  Otherwise, the original changelog will not be
          modified. (default)
   --no-force-save-on-release
-         Do not do so. Note that a dummy changelog entry made be supplied
+         Do not do so. Note that a dummy changelog entry may be supplied
          in order to achieve the same effect - e.g. $progname --release ""
          The entry will not be added to the changelog but its presence will
          suppress the editor
@@ -127,15 +128,15 @@ Options:
          Increment the Debian release number for a team upload
   --bpo
          Increment the Debian release number for a Backports.org upload
-	 to "lenny-backports"
+	 to "squeeze-backports"
   -l, --local <suffix>
          Add a suffix to the Debian version number for a local build
   -b, --force-bad-version
          Force a version to be less than the current one (e.g., when
          backporting)
-  --allow-lower-version
+  --allow-lower-version <pattern>
          Allow a version to be less than the current one (e.g., when
-         backporting) if it matches a specified pattern
+         backporting) if it matches the specified pattern
   --force-distribution
          Force the provided distribution to be used, even if it doesn't match
          the list of known distributions
@@ -151,7 +152,7 @@ Options:
   --no-preserve
          Do not preserve the directory name (default)
   --distributor <distributor>
-         Override the distributor ID from lsb_release.
+         Override the distributor ID from dpkg-vendor.
   -D, --distribution <dist>
          Use the specified distribution in the changelog entry being edited
   -u, --urgency <urgency>
@@ -170,6 +171,8 @@ Options:
          (default: do not)
   -m, --maintmaint
          Don\'t change (maintain) the maintainer details in the changelog entry
+  -M, --controlmaint
+         Use maintainer name and email from the debian/control Maintainer field
   -t, --mainttrailer
          Don\'t change (maintain) the trailer line in the changelog entry; i.e.
          maintain the maintainer and date/time details
@@ -218,6 +221,7 @@ my $check_dirname_level = 1;
 my $check_dirname_regex = 'PACKAGE(-.+)?';
 my $opt_p = 0;
 my $opt_query = 1;
+my $opt_distributor = undef;
 my $opt_release_heuristic = 'log';
 my $opt_multimaint = 1;
 my $opt_multimaint_merge = 0;
@@ -248,6 +252,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 		       'DEBCHANGE_LOWER_VERSION_PATTERN' => '',
 		       'DEBCHANGE_AUTO_NMU' => 'yes',
 		       'DEBCHANGE_FORCE_SAVE_ON_RELEASE' => 'yes',
+		       'DEBCHANGE_DISTRIBUTOR' => '',
 		       );
     $config_vars{'DEBCHANGE_TZ'} ||= '';
     my %config_default = %config_vars;
@@ -291,6 +296,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
     chomp $modified_conf_msg;
 
     $opt_p = $config_vars{'DEBCHANGE_PRESERVE'} eq 'yes' ? 1 : 0;
+    $opt_distributor = $config_vars{'DEBCHANGE_DISTRIBUTOR'};
     $opt_query = $config_vars{'DEBCHANGE_QUERY_BTS'} eq 'no' ? 0 : 1;
     $check_dirname_level = $config_vars{'DEVSCRIPTS_CHECK_DIRNAME_LEVEL'};
     $check_dirname_regex = $config_vars{'DEVSCRIPTS_CHECK_DIRNAME_REGEX'};
@@ -310,10 +316,9 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 # with older debchange versions.
 my ($opt_help, $opt_version);
 my ($opt_i, $opt_a, $opt_e, $opt_r, $opt_v, $opt_b, $opt_d, $opt_D, $opt_u, $opt_force_dist);
-my ($opt_n, $opt_bn, $opt_R, $opt_qa, $opt_s, $opt_team, $opt_U, $opt_bpo, $opt_l, $opt_c, $opt_m, $opt_create, $opt_package, @closes);
+my ($opt_n, $opt_bn, $opt_R, $opt_qa, $opt_s, $opt_team, $opt_U, $opt_bpo, $opt_l, $opt_c, $opt_m, $opt_M, $opt_create, $opt_package, @closes);
 my ($opt_news);
 my ($opt_level, $opt_regex, $opt_noconf, $opt_empty);
-my ($opt_distributor);
 
 Getopt::Long::Configure('bundling');
 GetOptions("help|h" => \$opt_help,
@@ -326,7 +331,7 @@ GetOptions("help|h" => \$opt_help,
 	   "package=s" => \$opt_package,
 	   "v|newversion=s" => \$opt_v,
 	   "b|force-bad-version" => \$opt_b,
-	   "allow-lower-version" => \$opt_allow_lower,
+	   "allow-lower-version=s" => \$opt_allow_lower,
 	   "force-distribution" => \$opt_force_dist,
 	   "d|fromdirname" => \$opt_d,
 	   "p" => \$opt_p,
@@ -352,6 +357,7 @@ GetOptions("help|h" => \$opt_help,
 	   'multimaint-merge!' => \$opt_multimaint_merge,
 	   'multi-maint-merge!' => \$opt_multimaint_merge,
 	   "m|maintmaint" => \$opt_m,
+	   "M|controlmaint" => \$opt_M,
 	   "t|mainttrailer!" => \$opt_t,
 	   "check-dirname-level=s" => \$opt_level,
 	   "check-dirname-regex=s" => \$opt_regex,
@@ -405,10 +411,10 @@ if (defined $opt_u) {
 
 # See if we're Debian, Ubuntu or someone else, if we can
 my $distributor;
-if (defined $opt_distributor) {
+if (not $opt_distributor eq '') {
     $distributor = $opt_distributor;
-} elsif (system('command -v lsb_release >/dev/null 2>&1') >> 8 == 0) {
-    $distributor = `lsb_release -is 2>/dev/null`;
+} elsif (system('command -v dpkg-vendor >/dev/null 2>&1') >> 8 == 0) {
+    $distributor = `dpkg-vendor --query Vendor 2>/dev/null`;
     chomp $distributor;
 }
 $distributor ||= 'Debian';
@@ -419,13 +425,13 @@ if ($distributor eq 'Ubuntu' and ($opt_n or $opt_bn or $opt_qa or $opt_bpo)) {
 # Check the distro name given.
 if (defined $opt_D) {
     if ($distributor eq 'Debian') {
-	unless ($opt_D =~ /^(unstable|((old)?stable|testing)(-security)?|experimental|UNRELEASED|(etch|lenny)-volatile|lenny-backports|((oldstable|testing)-)?proposed-updates)$/) {
-	    warn "$progname warning: Recognised distributions are: unstable, testing, stable,\noldstable, experimental, UNRELEASED, {etch,lenny}-volatile, lenny-backports,\n{oldstable-,testing-,}proposed-updates and {testing,stable,oldstable}-security.\nUsing your request anyway.\n";
+	unless ($opt_D =~ /^(unstable|((old)?stable|testing)(-security)?|experimental|UNRELEASED|lenny-volatile|lenny-backports(-sloppy)?|squeeze-backports|((oldstable|testing)-)?proposed-updates)$/) {
+	    warn "$progname warning: Recognised distributions are: unstable, testing, stable,\noldstable, experimental, UNRELEASED, lenny-volatile, lenny-backports{-sloppy,}, squeeze-backports,\n{oldstable-,testing-,}proposed-updates and {testing,stable,oldstable}-security.\nUsing your request anyway.\n";
 	    $warnings++ if not $opt_force_dist;
 	}
     } elsif ($distributor eq 'Ubuntu') {
-	unless ($opt_D =~ /^((dapper|hardy|jaunty|karmic|lucid|maverick|natty)(-updates|-security|-proposed|-backports)?|UNRELEASED)$/) {
-	    warn "$progname warning: Recognised distributions are:\n{dapper,hardy,jaunty,karmic,lucid,maverick,natty}{,-updates,-security,-proposed,-backports} and UNRELEASED.\nUsing your request anyway.\n";
+	unless ($opt_D =~ /^((hardy|lucid|maverick|natty|oneiric|precise|quantal)(-updates|-security|-proposed|-backports)?|UNRELEASED)$/) {
+	    warn "$progname warning: Recognised distributions are:\n{hardy,lucid,maverick,natty,oneiric,precise,quantal}{,-updates,-security,-proposed,-backports} and UNRELEASED.\nUsing your request anyway.\n";
 	    $warnings++ if not $opt_force_dist;
 	}
     } else {
@@ -479,7 +485,7 @@ if (! $opt_create) {
 		fatal "Cannot find $changelog_path anywhere!\nAre you in the source code tree?\n(You could use --create if you wish to create this file.)";
 	    }
 	}
-	
+
 	# Can't write, so stop now.
 	if (! -w $changelog_path) {
 	    fatal "$changelog_path is not writable!";
@@ -521,8 +527,8 @@ my $MAINTAINER = 'MAINTAINER';
 my $EMAIL = 'EMAIL';
 my $DISTRIBUTION = 'UNRELEASED';
 my $bpo_dist = '';
-my %bpo_dists = ( '40', 'etch', '50', 'lenny' );
-my $latest_bpo_dist = '50';
+my %bpo_dists = ( 50, 'lenny', 60, 'squeeze' );
+my $latest_bpo_dist = '60';
 my $CHANGES = '';
 # Changelog urgency, possibly propogated to NEWS files
 my $CL_URGENCY = '';
@@ -568,8 +574,8 @@ if (! $opt_create || ($opt_create && $opt_news)) {
     fatal "No distribution in changelog!"
 	unless exists $changelog{'Distribution'};
     if ($distributor eq 'Ubuntu') {
-	# In Ubuntu uploads should go to maverick
-	$DISTRIBUTION = 'maverick';
+	# In Ubuntu uploads should go to precise
+	$DISTRIBUTION = 'precise';
     } else {
 	$DISTRIBUTION=$changelog{'Distribution'};
     }
@@ -610,7 +616,7 @@ if (! $opt_create || ($opt_create && $opt_news)) {
 	my $gooddir;
 	if ($re =~ m%/%) { $gooddir = eval "cwd() =~ /^$re\$/;"; }
 	else { $gooddir = eval "basename(cwd()) =~ /^$re\$/;"; }
-	
+
 	if (! $gooddir) {
 	    my $pwd = cwd();
 	    fatal <<"EOF";
@@ -691,7 +697,7 @@ if (! exists $env{'DEBEMAIL'} or ! exists $env{'DEBFULLNAME'}) {
 }
 
 # Now use the gleaned values to detemine our MAINTAINER and EMAIL values
-if (! $opt_m) {
+if (! $opt_m and ! $opt_M) {
     if (exists $env{'DEBFULLNAME'}) {
 	$MAINTAINER = $env{'DEBFULLNAME'};
     } elsif (exists $env{'NAME'}) {
@@ -736,13 +742,33 @@ if (! $opt_m) {
 	$EMAIL = $addr if $addr;
     }
     # Otherwise, $EMAIL retains its default value of the last changelog entry
-} # if (! $opt_m)
+} # if (! $opt_m and ! $opt_M)
+
+if ($opt_M) {
+    if (-f 'debian/control') {
+	if (have_lpdc()) {
+	    my $parser = Parse::DebControl->new;
+	    my $deb822 = $parser->parse_file('debian/control', {stripComments => 'true'});
+	    my $maintainer = decode_utf8($deb822->[0]->{'Maintainer'});
+	    if ($maintainer =~ /^(.*)\s+<(.*)>$/) {
+		$MAINTAINER = $1;
+		$EMAIL = $2;
+	    } else {
+		fatal "$progname: invalid debian/control Maintainer field value\n";
+	    }
+	} else {
+	    fatal "$progname: unable to get maintainer from debian/control: $lpdc_broken\n";
+	}
+    } else {
+	fatal "Missing file debian/control";
+    }
+}
 
 #####
 
-if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and 
+if ($opt_auto_nmu eq 'yes' and ! $opt_v and ! $opt_l and ! $opt_s and
     ! $opt_team and ! $opt_qa and ! $opt_bpo and ! $opt_bn and ! $opt_R and ! $opt_n and ! $opt_c and
-    ! (exists $ENV{'CHANGELOG'} and length $ENV{'CHANGELOG'}) and
+    ! (exists $ENV{'CHANGELOG'} and length $ENV{'CHANGELOG'}) and ! $opt_M and
     ! $opt_create and ! $opt_a_passed and ! $opt_r and ! $opt_e and
     $distributor ne 'Ubuntu' and
     ! ($opt_release_heuristic eq 'changelog' and
@@ -1240,7 +1266,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
     my $multimaint=0;
     if (! $opt_news) {
 	my $lastmultimaint;
-	
+
 	# Parse the changelog for multi-maintainer maintainer lines of
 	# the form [ Full Name ] and record the last of these.
 	while ($CHANGES=~/.*\n^\s+\[\s+([^\]]+)\s+]\s*$/mg) {
@@ -1255,7 +1281,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
 	    (defined $nextmaint)
 	   ) {
 	    $multimaint=1;
-	
+
 	    if (! $lastmultimaint) {
 		# Add a multi-maintainer header to the top of the existing
 		# changelog.
@@ -1280,7 +1306,7 @@ if (($opt_r || $opt_a || $merge) && ! $opt_create) {
 		    $distribution = $dist_indicator;
 		} else {
 		    if ($distributor eq 'Ubuntu') {
-			$distribution = $opt_D || "maverick";
+			$distribution = $opt_D || "precise";
 		    } else {
 			$distribution = $opt_D || $lastdist || "unstable";
 		    }
@@ -1440,16 +1466,14 @@ if ((!$TEXT and !$EMPTY_TEXT and ! ($opt_create and $opt_empty)) or @closes_text
     system("sensible-editor +$line $changelog_path.dch") == 0 or
 	fatal "Error editing $changelog_path";
 
-    if (! @closes_text) { # so must have a changelog added by hand
-	my $newmtime = (stat("$changelog_path.dch"))[9];
-	defined $newmtime or fatal
-	    "Error getting modification time of temporary $changelog_path: $!";
-	if ($mtime == $newmtime && ! $opt_create &&
-	    (!$opt_r || ($opt_r && $opt_force_save_on_release))) {
+    my $newmtime = (stat("$changelog_path.dch"))[9];
+    defined $newmtime or fatal
+	"Error getting modification time of temporary $changelog_path: $!";
+    if ($mtime == $newmtime && ! $opt_create &&
+	(!$opt_r || ($opt_r && $opt_force_save_on_release))) {
 
-	    warn "$progname: $changelog_path unmodified; exiting.\n";
-	    exit 0;
-	}
+	warn "$progname: $changelog_path unmodified; exiting.\n";
+	exit 0;
     }
 }
 
@@ -1490,7 +1514,7 @@ if ((basename(cwd()) =~ m%^\Q$PACKAGE\E-\Q$UVERSION\E$%) &&
 	}
 	# And check whether a new orig tarball exists
 	my @origs = glob("../$PACKAGE\_$new_uversion.*");
-	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.(gz|bz2|lzma|xz)$/ } @origs;
+	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.$compression_re_file_ext$/ } @origs;
 	if ($num_origs == 0) {
 	    warn "$progname warning: no orig tarball found for the new version.\n";
 	}
@@ -1499,21 +1523,23 @@ if ((basename(cwd()) =~ m%^\Q$PACKAGE\E-\Q$UVERSION\E$%) &&
 
 exit 0;
 
-
-# Format for standard Debian changelogs
-format CHANGELOG =
+{
+    no warnings 'uninitialized';
+    # Format for standard Debian changelogs
+    format CHANGELOG =
   * ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
     $CHGLINE
  ~~ ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
     $CHGLINE
 .
-# Format for NEWS files.
-format NEWS =
+    # Format for NEWS files.
+    format NEWS =
   ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
     $CHGLINE
 ~~^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
     $CHGLINE
 .
+}
 
 my $linecount=0;
 sub format_line {
