@@ -1,4 +1,4 @@
-#!/usr/bin/perl -w
+#!/usr/bin/perl
 #
 # debcheckout: checkout the development repository of a Debian package
 # Copyright (C) 2007-2009  Stefano Zacchiroli <zack@debian.org>
@@ -47,8 +47,8 @@ directory; this can be overridden by providing the I<DESTDIR> argument.
 
 The information about where the repository is available is expected to be found
 in B<Vcs-*> fields available in the source package record. For example, the B<vim>
-package exposes such information with a field like S<B<Vcs-Git:
-git://git.debian.org/git/pkg-vim/vim.git>>, you can see it by grepping through
+package exposes such information with a field like S<B<Vcs-Hg:
+http://hg.debian.org/hg/pkg-vim/vim>>, you can see it by grepping through
 B<apt-cache showsrc vim>.
 
 If more than one source package record containing B<Vcs-*> fields is available,
@@ -223,12 +223,13 @@ B<debcheckout> and this manpage have been written by Stefano Zacchiroli
 use feature 'switch';
 use strict;
 use warnings;
-use Getopt::Long;
+use Getopt::Long qw(:config gnu_getopt);
 use Pod::Usage;
 use File::Basename;
 use File::Copy qw/copy/;
 use File::Temp qw/tempdir/;
 use Cwd;
+use Dpkg::Compression;
 use lib '/usr/share/devscripts';
 use Devscripts::Versort;
 
@@ -322,7 +323,7 @@ sub find_repo($$) {
 	    ($type, $url) = (lc($2), $3);
 	} elsif ($line =~ /^Version:\s*(.*)$/i) {
 	    $version = $1;
-	} elsif ($line =~ /^ [a-f0-9]{32} \d+ (\S+)\.orig\.tar\.gz$/) {
+	} elsif ($line =~ /^ [a-f0-9]{32} \d+ (\S+)\.orig\.tar\.$compression_re_file_ext$/) {
 	    $origtgz_name = $1;
 	} elsif ($line =~ /^$/) {
 	    push (@repos, [$version, $type, $url, $origtgz_name])
@@ -854,8 +855,8 @@ sub unpack_source($$$$$) {
 	} elsif ($file eq '.' or $file eq '..') {
 	    next;
 	} else {
-	    rename "$directory/$file", "$pkg/$file" or
-		die "rename $directory/$file $pkg/$file: $!";
+	    rename "$directory/$file", "$destdir/$file" or
+		die "rename $directory/$file $destdir/$file: $!";
 	}
     }
     closedir DIR;
