@@ -18,8 +18,9 @@ use 5.006_000;
 use strict;
 use Cwd;
 use Dpkg::IPC;
+use Dpkg::Compression;
+use File::Copy qw(cp move);
 use File::Basename;
-use File::Copy qw/ cp move /;
 use File::Path qw/ rmtree /;
 use File::Temp qw/ tempdir tempfile /;
 use lib '/usr/share/devscripts';
@@ -499,7 +500,7 @@ elsif ($type eq 'dsc') {
 		if ($file =~ /\.diff\.gz$/) {
 		    $diffs[$i] = cwd() . '/' . $file;
 		}
-		elsif ($file =~ /(\.orig)?\.tar\.(gz|bz2|lzma|xz)$/) {
+		elsif ($file =~ /(?:\.orig)?\.tar\.$compression_re_file_ext$/) {
 		    $origs[$i] = $file;
 		}
 	    } else {
@@ -616,28 +617,14 @@ elsif ($type eq 'dsc') {
 	    while ($_ = readdir(DIR)) {
 		    my $unpacked = "=unpacked-tar" . $tarballs . "=";
 		    my $filename = $_;
-		    my $found = 0;
-		    my $comp = "";
-
-		    if ($_ =~ /tar.gz$/) {
-			$filename =~ s%(.*)\.tar\.gz$%$1%;
+		    if ($filename =~ s/\.tar\.$compression_re_file_ext$//) {
+			my $comp = compression_guess_from_filename($_);
 			$tarballs++;
-			$found = 1;
-			$comp = "gzip";
-		    }
-		    if ($_ =~ /tar.bz$/ || $_ =~ /tar.bz2$/) {
-			$filename =~ s%(.*)\.tar\.bz2?$%$1%;
-			$tarballs++;
-			$found = 1;
-			$comp = "bzip2";
-		    }
-
-		    if ($found) {
 			spawn(exec => ['tar', "--$comp", '-xf', $_],
-			    to_file => '/dev/null',
-			    wait_child => 1,
-			    chdir => "$diri/$sdiri",
-			    nocheck => 1);
+			      to_file => '/dev/null',
+			      wait_child => 1,
+			      chdir => "$diri/$sdiri",
+			      nocheck => 1);
 			if (-d "$diri/$sdiri/$filename") {
 			    move "$diri/$sdiri/$filename", "$diri/$sdiri/$unpacked";
 			}
@@ -1045,7 +1032,7 @@ sub wdiff_control_files($$$$$)
 	    } elsif ($wdiff_opt) {
 		# Don't try messing with control codes
 		my $msg = ucfirst($cf) . " files$usepkgname: wdiff output";
-		print $msg, "\n", '-' x length $msg, "\n";
+		print "\n", $msg, "\n", '-' x length $msg, "\n";
 		print $wdiff;
 		$status = 1;
 	    } else {
@@ -1054,7 +1041,7 @@ sub wdiff_control_files($$$$$)
 		@output = grep /(\[-|\{\+)/, @output;
 		my $msg = ucfirst($cf) . " files$usepkgname: lines which differ (wdiff format)";
 		print "\n", $msg, "\n", '-' x length $msg, "\n";
-                print join("\n",@output), "\n";
+		print join("\n",@output), "\n";
 		$status = 1;
 	    }
 	}
