@@ -31,7 +31,8 @@ use Cwd qw(abs_path);
 use IO::Dir;
 use IO::File;
 use Digest::MD5;
-use Getopt::Long;
+use Dpkg::Compression;
+use Getopt::Long qw(:config gnu_getopt);
 use File::Basename;
 
 # global variables
@@ -198,7 +199,8 @@ sub get_file {
     }
 
     # try apt-get if it is still not there
-    if (not -e $file and $file =~ m!^([a-z0-9.+-]{2,})_[^/]+\.(?:diff\.gz|tar\.gz)$!) {
+    my $ext = $compression_re_file_ext;
+    if (not -e $file and $file =~ m!^([a-z0-9.+-]{2,})_[^/]+\.(?:diff|tar)\.$ext$!) {
 	my @cmd = ('apt-get', 'source', '--print-uris', $1);
 	my $cmd = join ' ', @cmd;
 	open(my $apt, '-|', @cmd) or die "$cmd: $!";
@@ -320,7 +322,8 @@ sub apt_get {
 
     # find deb lines matching the hosts in the policy output
     my @repositories;
-    my $host_re = '(?:' . (join '|', map { quotemeta; } @hosts) . ')';
+    # the regexp within the map below can be removed and replaced with only the quotemeta statement once bug #154868 is fixed
+    my $host_re = '(?:' . (join '|', map { my $host = quotemeta; $host =~ s/(?<=\\:\\\/\\\/)([^\\:]+)(?=\\\/)/$1(?::[0-9]+\)?/g; $host; } @hosts) . ')';
     if (-f "/etc/apt/sources.list") {
 	$apt = new IO::File("/etc/apt/sources.list") or die "/etc/apt/sources.list: $!";
 	while (<$apt>) {
