@@ -19,17 +19,18 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 use 5.008;  # uses 'our' variables and filetest
 use strict;
 use Cwd;
+use Cwd 'abs_path';
+use Dpkg::IPC;
 use File::Basename;
 use File::Copy;
-use File::Temp qw/tempdir/;
+use File::Temp qw/tempfile tempdir/;
 use filetest 'access';
-use Getopt::Long;
+use Getopt::Long qw(:config gnu_getopt);
 use lib '/usr/share/devscripts';
 use Devscripts::Versort;
 use Text::ParseWords;
@@ -92,9 +93,10 @@ Options:
                    servers to respond (default 20 seconds)
     --symlink      Make an orig.tar.gz symlink to downloaded file (default)
     --rename       Rename to orig.tar.gz instead of symlinking
-                   (Both will use orig.tar.bz2 if appropriate)
-    --repack       Repack downloaded archives from orig.tar.bz2 or orig.zip to
-                   orig.tar.gz
+                   (Both will use orig.tar.bz2, orig.tar.lzma, or orig.tar.xz
+                   if appropriate)
+    --repack       Repack downloaded archives from orig.tar.bz2, orig.tar.lzma,
+                   orig.tar.xz or orig.zip to orig.tar.gz
                    (does nothing if downloaded archive orig.tar.gz)
     --no-symlink   Don\'t make symlink or rename
     --verbose      Give verbose output
@@ -164,7 +166,7 @@ my $download = 1;
 my $download_version;
 my $force_download = 0;
 my $report = 0; # report even on up-to-date packages?
-my $repack = 0; # repack .tar.bz2 or .zip to .tar.gz
+my $repack = 0; # repack .tar.bz2, .tar.lzma, .tar.xz or .zip to .tar.gz
 my $symlink = 'symlink';
 my $verbose = 0;
 my $check_dirname_level = 1;
@@ -419,7 +421,7 @@ if (defined $opt_watchfile) {
 	if (! defined $package || ! defined $debversion) {
 	    die "$progname: Problems determining package name and/or version from\n  debian/changelog\n";
 	}
-	
+
 	# Check the directory is properly named for safety
 	my $good_dirname = 1;
 	if ($check_dirname_level ==  2 or
@@ -612,12 +614,12 @@ exit ($found ? 0 : 1);
 # watch_version=2:
 #
 # For ftp sites:
-#   ftp://site.name/dir/path/pattern-(.*)\.tar\.gz [version [action]]
+#   ftp://site.name/dir/path/pattern-(.+)\.tar\.gz [version [action]]
 #
 # For http sites:
-#   http://site.name/dir/path/pattern-(.*)\.tar\.gz [version [action]]
+#   http://site.name/dir/path/pattern-(.+)\.tar\.gz [version [action]]
 # or
-#   http://site.name/dir/path/base pattern-(.*)\.tar\.gz [version [action]]
+#   http://site.name/dir/path/base pattern-(.+)\.tar\.gz [version [action]]
 #
 # Lines can be prefixed with opts=<opts>.
 #
@@ -628,51 +630,51 @@ exit ($found ? 0 : 1);
 # watch_version=3:
 #
 # Correct handling of regex special characters in the path part:
-# ftp://ftp.worldforge.org/pub/worldforge/libs/Atlas-C++/transitional/Atlas-C\+\+-(.*)\.tar\.gz
-# 
+# ftp://ftp.worldforge.org/pub/worldforge/libs/Atlas-C++/transitional/Atlas-C\+\+-(.+)\.tar\.gz
+#
 # Directory pattern matching:
 # ftp://ftp.nessus.org/pub/nessus/nessus-([\d\.]+)/src/nessus-core-([\d\.]+)\.tar\.gz
-# 
+#
 # The pattern in each part may contain several (...) groups and
 # the version number is determined by joining all groups together
 # using "." as separator.  For example:
 #   ftp://site/dir/path/pattern-(\d+)_(\d+)_(\d+)\.tar\.gz
-# 
+#
 # This is another way of handling site with funny version numbers,
 # this time using mangling.  (Note that multiple groups will be
 # concatenated before mangling is performed, and that mangling will
 # only be performed on the basename version number, not any path version
 # numbers.)
 # opts=uversionmangle=s/^/0.0./ \
-#   ftp://ftp.ibiblio.org/pub/Linux/ALPHA/wine/development/Wine-(.*)\.tar\.gz
-# 
+#   ftp://ftp.ibiblio.org/pub/Linux/ALPHA/wine/development/Wine-(.+)\.tar\.gz
+#
 # Similarly, the upstream part of the Debian version number can be
 # mangled:
 # opts=dversionmangle=s/\.dfsg\.\d+$// \
-#   http://some.site.org/some/path/foobar-(.*)\.tar\.gz
-# 
+#   http://some.site.org/some/path/foobar-(.+)\.tar\.gz
+#
 # The versionmangle=... option is a shorthand for saying uversionmangle=...
 # and dversionmangle=... and applies to both upstream and Debian versions.
-# 
+#
 # The option filenamemangle can be used to mangle the name under which
 # the downloaded file will be saved:
 #   href="http://foo.bar.org/download/?path=&amp;download=foo-0.1.1.tar.gz"
 # could be handled as:
 # opts=filenamemangle=s/.*=(.*)/$1/ \
-#     http://foo.bar.org/download/\?path=&amp;download=foo-(.*)\.tar\.gz
+#     http://foo.bar.org/download/\?path=&amp;download=foo-(.+)\.tar\.gz
 # and
 #   href="http://foo.bar.org/download/?path=&amp;download_version=0.1.1"
 # as:
 # opts=filenamemangle=s/.*=(.*)/foo-$1\.tar\.gz/ \
-#    http://foo.bar.org/download/\?path=&amp;download_version=(.*)
-# 
+#    http://foo.bar.org/download/\?path=&amp;download_version=(.+)
+#
 # The option downloadurlmangle can be used to mangle the URL of the file
 # to download.  This can only be used with http:// URLs.  This may be
 # necessary if the link given on the webpage needs to be transformed in
 # some way into one which will work automatically, for example:
 # opts=downloadurlmangle=s/prdownload/download/ \
 #   http://developer.berlios.de/project/showfiles.php?group_id=2051 \
-#   http://prdownload.berlios.de/softdevice/vdr-softdevice-(.*).tgz
+#   http://prdownload.berlios.de/softdevice/vdr-softdevice-(.+).tgz
 
 
 sub process_watchline ($$$$$$)
@@ -697,7 +699,7 @@ sub process_watchline ($$$$$$)
     %dehs_tags = ('package' => $pkg);
 
     if ($watch_version == 1) {
-	($site, $dir, $pattern, $lastversion, $action) = split ' ', $line, 5;
+	($site, $dir, $filepattern, $lastversion, $action) = split ' ', $line, 5;
 
 	if (! defined $lastversion or $site =~ /\(.*\)/ or $dir =~ /\(.*\)/) {
 	    warn "$progname warning: there appears to be a version 2 format line in\n  the version 1 watchfile $watchfile;\n  Have you forgotten a 'version=2' line at the start, perhaps?\n  Skipping the line: $line\n";
@@ -705,15 +707,15 @@ sub process_watchline ($$$$$$)
 	}
 	if ($site !~ m%\w+://%) {
 	    $site = "ftp://$site";
-	    if ($pattern !~ /\(.*\)/) {
+	    if ($filepattern !~ /\(.*\)/) {
 		# watch_version=1 and old style watchfile;
 		# pattern uses ? and * shell wildcards; everything from the
 		# first to last of these metachars is the pattern to match on
-		$pattern =~ s/(\?|\*)/($1/;
-		$pattern =~ s/(\?|\*)([^\?\*]*)$/$1)$2/;
-		$pattern =~ s/\./\\./g;
-		$pattern =~ s/\?/./g;
-		$pattern =~ s/\*/.*/g;
+		$filepattern =~ s/(\?|\*)/($1/;
+		$filepattern =~ s/(\?|\*)([^\?\*]*)$/$1)$2/;
+		$filepattern =~ s/\./\\./g;
+		$filepattern =~ s/\?/./g;
+		$filepattern =~ s/\*/.*/g;
 		$style='old';
 		warn "$progname warning: Using very old style of filename pattern in $watchfile\n  (this might lead to incorrect results): $3\n";
 	    }
@@ -724,6 +726,7 @@ sub process_watchline ($$$$$$)
 	$base =~ s%(?<!:)//%/%g;
 	$base =~ m%^(\w+://[^/]+)%;
 	$site = $1;
+	$pattern = $filepattern;
     } else {
 	# version 2/3 watchfile
 	if ($line =~ s/^opt(?:ion)?s=//) {
@@ -784,7 +787,7 @@ sub process_watchline ($$$$$$)
 	}
 
 	# Check all's OK
-	if ($filepattern !~ /\(.*\)/) {
+	if (not $filepattern or $filepattern !~ /\(.*\)/) {
 	    warn "$progname warning: Filename pattern missing version delimiters ()\n  in $watchfile, skipping:\n  $line\n";
 	    return 1;
 	}
@@ -870,13 +873,13 @@ sub process_watchline ($$$$$$)
 	}
 
 	@redirections = @{$user_agent->get_redirections};
-	
+
 	print STDERR "$progname debug: redirections: @redirections\n"
 	    if $debug;
 
 	foreach my $_redir (@redirections) {
 	    my $base_dir = $_redir;
-	    
+
 	    $base_dir =~ s%^\w+://[^/]+/%/%;
 	    if ($_redir =~ m%^(\w+://[^/]+)%) {
 		my $base_site = $1;
@@ -1007,7 +1010,7 @@ sub process_watchline ($$$$$$)
 	# We separate out HTMLised listings from standard listings, so
 	# that we can target our search correctly
 	if ($content =~ /<\s*a\s+[^>]*href/i) {
-	    while ($content =~ 
+	    while ($content =~
 		m/(?:<\s*a\s+[^>]*href\s*=\s*\")((?-i)$pattern)\"/gi) {
 		my $file = $1;
 		my $mangled_version = join(".", $file =~ m/^$pattern$/);
@@ -1026,22 +1029,24 @@ sub process_watchline ($$$$$$)
 	} else {
 	    # they all look like:
 	    # info info ... info filename [ -> linkname]
-	    while ($content =~ m/\s($filepattern)(\s+->\s+\S+)?$/mg) {
-		my $file = $1;
-		my $mangled_version = join(".", $file =~ m/^$filepattern$/);
-		foreach my $pat (@{$options{'uversionmangle'}}) {
-		    if (! safe_replace(\$mangled_version, $pat)) {
-			warn "$progname: In $watchfile, potentially"
-			  . " unsafe or malformed uversionmangle"
-			  . " pattern:\n  '$pat'"
-			  . " found. Skipping watchline\n"
-			  . "  $line\n";
-			return 1;
+	    for my $ln (split(/\n/, $content)) {
+		if ($ln =~ m/\s($filepattern)(\s+->\s+\S+)?$/) {
+		    my $file = $1;
+		    my $mangled_version = join(".", $file =~ m/^$filepattern$/);
+		    foreach my $pat (@{$options{'uversionmangle'}}) {
+			if (! safe_replace(\$mangled_version, $pat)) {
+			    warn "$progname: In $watchfile, potentially"
+			      . " unsafe or malformed uversionmangle"
+			      . " pattern:\n  '$pat'"
+			      . " found. Skipping watchline\n"
+			      . "  $line\n";
+			    return 1;
+			}
 		    }
+		    push @files, [$mangled_version, $file];
 		}
-		push @files, [$mangled_version, $file];
 	    }
-	}	    
+	}
 
 	if (@files) {
 	    if ($verbose) {
@@ -1110,7 +1115,7 @@ EOF
 	    $newfile_base = "$pkg-$newversion.download";
 	}
     }
-    
+
     # So what have we got to report now?
     my $upstream_url;
     # Upstream URL?  Copying code from below - ugh.
@@ -1119,6 +1124,11 @@ EOF
 	if ($newfile =~ m%^\w+://%) {
 	    $upstream_url = $newfile;
 	}
+	elsif ($newfile =~ m%^//%) {
+	    $upstream_url = $site;
+	    $upstream_url =~ s/^(https?:).*/$1/;
+	    $upstream_url .= $newfile;
+	}
 	# absolute filename?
 	elsif ($newfile =~ m%^/%) {
 	    # Were there any redirections? If so try using those first
@@ -1242,7 +1252,7 @@ EOF
     }
 
     if (defined $pkg_dir) {
-    	if (! -d "$destdir") {
+	if (! -d "$destdir") {
 	    print "Package directory '$destdir to store downloaded file is not existing\n";
 	    return 1;
 	}
@@ -1251,15 +1261,12 @@ EOF
 		if $verbose or ($download == 0 and ! $dehs);
 	    return 0;
 	}
-	if (-f "$destdir/${pkg}_${newversion}.orig.tar.gz") {
-	    print " => ${pkg}_${newversion}.orig.tar.gz already in package directory '$destdir'\n"
-		if $verbose or ($download == 0 and ! $dehs);
-	    return 0;
-	}
-	elsif (-f "$destdir/${pkg}_${newversion}.orig.tar.bz2") {
-	    print " => ${pkg}_${newversion}.orig.tar.bz2 already in package directory '$destdir'\n"
-		if $verbose or ($download == 0 and ! $dehs);
-	    return 0;
+	foreach my $suffix (qw(gz bz2 lzma xz)) {
+	    if (-f "$destdir/${pkg}_${newversion}.orig.tar.$suffix") {
+		print " => ${pkg}_${newversion}.orig.tar.$suffix already in package directory '$destdir'\n"
+		    if $verbose or ($download == 0 and ! $dehs);
+		return 0;
+	    }
 	}
     }
 
@@ -1338,8 +1345,14 @@ EOF
     if ($repack and $newfile_base =~ /^(.*)\.(tar\.bz2|tbz2?)$/) {
 	print "-- Repacking from bzip2 to gzip\n" if $verbose;
 	my $newfile_base_gz = "$1.tar.gz";
-	system("bunzip2 -c $destdir/$newfile_base | gzip -n -9 > $destdir/$newfile_base_gz") == 0
-	  or die "repacking from bzip2 to gzip failed\n";
+	my (undef, $fname) = tempfile(UNLINK => 1);
+	spawn(exec => ['bunzip2', '-c', "$destdir/$newfile_base"],
+	      to_file => $fname,
+	      wait_child => 1);
+	spawn(exec => ['gzip', '-n', '-9'],
+	      from_file => $fname,
+	      to_file => "$destdir/$newfile_base_gz",
+	      wait_child => 1);
 	unlink "$destdir/$newfile_base";
 	$newfile_base = $newfile_base_gz;
     }
@@ -1347,8 +1360,14 @@ EOF
     if ($repack and $newfile_base =~ /^(.*)\.(tar\.lzma|tlz(?:ma?)?)$/) {
 	print "-- Repacking from lzma to gzip\n" if $verbose;
 	my $newfile_base_gz = "$1.tar.gz";
-	system("lzma -cd $destdir/$newfile_base | gzip -n -9 > $destdir/$newfile_base_gz") == 0
-	  or die "repacking from lzma to gzip failed\n";
+	my (undef, $fname) = tempfile(UNLINK => 1);
+	spawn(exec => ['xz', '-F', 'lzma', '-cd', "$destdir/$newfile_base"],
+	      to_file => $fname,
+	      wait_child => 1);
+	spawn(exec => ['gzip', '-n', '-9'],
+	      from_file => $fname,
+	      to_file => "$destdir/$newfile_base_gz",
+	      wait_child => 1);
 	unlink "$destdir/$newfile_base";
 	$newfile_base = $newfile_base_gz;
     }
@@ -1356,8 +1375,14 @@ EOF
     if ($repack and $newfile_base =~ /^(.*)\.(tar\.xz|txz)$/) {
 	print "-- Repacking from xz to gzip\n" if $verbose;
 	my $newfile_base_gz = "$1.tar.gz";
-	system("xz -cd $destdir/$newfile_base | gzip -n -9 > $destdir/$newfile_base_gz") == 0
-	  or die "repacking from xz to gzip failed\n";
+	my (undef, $fname) = tempfile(UNLINK => 1);
+	spawn(exec => ['xz', '-cd', "$destdir/$newfile_base"],
+	      to_file => $fname,
+	      wait_child => 1);
+	spawn(exec => ['gzip', '-n', '-9'],
+	      from_file => $fname,
+	      to_file => "$destdir/$newfile_base_gz",
+	      wait_child => 1);
 	unlink "$destdir/$newfile_base";
 	$newfile_base = $newfile_base_gz;
     }
@@ -1369,99 +1394,99 @@ EOF
 	  or die("unzip binary not found. You need to install the package unzip to be able to repack .zip upstream archives.\n");
 
 	my $newfile_base_gz = "$1.tar.gz";
-	my $tempdir = tempdir ( "uscanXXXX", TMPDIR => 1, CLEANUP => 1 );
-	system("unzip -q -d $tempdir $destdir/$newfile_base; GZIP=-9 tar -C $tempdir -czf $destdir/$newfile_base_gz .") == 0 
-	  or die("Repacking from zip to tar.gz failed\n");
+	my $tempdir = tempdir ("uscanXXXX", TMPDIR => 1, CLEANUP => 1);
+	# Parent of the target directory should be under our control
+	$tempdir .= '/repack';
+	mkdir $tempdir or die("Unable to mkdir($tempdir): $!\n");
+	my $absdestdir = abs_path($destdir);
+	system('unzip', '-q', '-a', '-d', $tempdir, "$destdir/$newfile_base") == 0
+	    or die("Repacking from zip to tar.gz failed (could not unzip)\n");
+	my $cwd = cwd();
+	chdir($tempdir) or die("Unable to chdir($tempdir): $!\n");
+	eval {
+	    spawn(exec => ['tar', '--owner=root', '--group=root', '--mode=a+rX', '-czf', "$absdestdir/$newfile_base_gz", glob('* .[!.]*')],
+		  env => { GZIP => '-n -9' },
+		  wait_child => 1);
+	};
+	if ($@) {
+	    die("Repacking from zip to tar.gz failed (could not create tarball)\n");
+	}
+	chdir($cwd);
 	unlink "$destdir/$newfile_base";
 	$newfile_base = $newfile_base_gz;
     }
 
-    if ($newfile_base =~ /\.(tar\.gz|tgz|tar\.bz2|tbz2?)$/) {
-	my $filetype = `file $destdir/$newfile_base`;
-	$filetype =~ s%^\.\./\Q$newfile_base\E: %%;
-	unless ($filetype =~ /compressed data/) {
+    if ($newfile_base =~ /\.(tar\.gz|tgz
+			     |tar\.bz2|tbz2?
+			     |tar.lzma|tlz(?:ma?)?
+			     |tar.xz|txz)$/x) {
+	my $filetype;
+	eval {
+	    spawn(exec => ['file', '-b', '-k', "$destdir/$newfile_base"],
+		  to_string => \$filetype,
+		  wait_child => 1);
+	};
+	unless (defined $filetype && $filetype =~ /compressed data/) {
 	    warn "$progname warning: $destdir/$newfile_base does not appear to be a compressed file;\nthe file command says: $filetype\nNot processing this file any further!\n";
 	    return 1;
 	}
     }
 
-    if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
-	if ($symlink eq 'symlink') {
-	    symlink $newfile_base, "$destdir/${pkg}_${newversion}.orig.tar.gz";
-	} elsif ($symlink eq 'rename') {
-	    move "$destdir/$newfile_base", "$destdir/${pkg}_${newversion}.orig.tar.gz";
-	}
-    }
-    elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
-	if ($symlink eq 'symlink') {
-	    symlink $newfile_base, "$destdir/${pkg}_${newversion}.orig.tar.bz2";
-	} elsif ($symlink eq 'rename') {
-	    move "$destdir/$newfile_base", "$destdir/${pkg}_${newversion}.orig.tar.bz2";
-	}
-    }
+    my @renames = (
+	[qr/\.(tar\.gz|tgz)$/, 'gz'],
+	[qr/\.(tar\.bz2|tbz2?)$/, 'bz2'],
+	[qr/\.tar\.lzma|tlz(?:ma?)?$/, 'lzma'],
+	[qr/\.(tar\.xz|txz)$/, 'xz'],
+    );
 
-    if ($verbose) {
-	print "-- Successfully downloaded updated package $newfile_base\n";
-	if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
+    my ($renamed_base);
+    foreach my $pair (@renames) {
+	if ($newfile_base !~ $pair->[0]) {
+	    next;
+	}
+
+	my ($pattern, $suffix) = @{$pair};
+	$renamed_base = "${pkg}_${newversion}.orig.tar.$suffix";
+	if ($symlink eq 'symlink') {
+	    symlink $newfile_base, "$destdir/$renamed_base";
+	} elsif ($symlink eq 'rename') {
+	    move "$destdir/$newfile_base", "$destdir/$renamed_base";
+	}
+	if ($verbose) {
+	    print "-- Successfully downloaded updated package $newfile_base\n";
 	    if ($symlink eq 'symlink') {
-		print "    and symlinked ${pkg}_${newversion}.orig.tar.gz to it\n";
+		print "    and symlinked $renamed_base to it\n";
 	    } elsif ($symlink eq 'rename') {
-		print "    and renamed it as ${pkg}_${newversion}.orig.tar.gz\n";
+		print "    and renamed it as $renamed_base\n";
 	    }
-	}
-	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
+	} elsif ($dehs) {
+	    my $msg = "Successfully downloaded updated package $newfile_base";
+	    $dehs_tags{'target'} = "$renamed_base";
 	    if ($symlink eq 'symlink') {
-		print "    and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it\n";
+		$msg .= " and symlinked $renamed_base to it";
 	    } elsif ($symlink eq 'rename') {
-		print "    and renamed it as ${pkg}_${newversion}.orig.tar.bz2\n";
+		$msg .= " and renamed it as $renamed_base";
+	    } else {
+		$dehs_tags{'target'} = $newfile_base;
 	    }
-	}
-    } elsif ($dehs) {
-	my $msg = "Successfully downloaded updated package $newfile_base";
-	if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
+	    dehs_msg($msg);
+	} else {
+	    print "$pkg: Successfully downloaded updated package $newfile_base\n";
 	    if ($symlink eq 'symlink') {
-		$msg .= " and symlinked ${pkg}_${newversion}.orig.tar.gz to it";
+		print "    and symlinked $renamed_base to it\n";
 	    } elsif ($symlink eq 'rename') {
-		$msg .= " and renamed it as ${pkg}_${newversion}.orig.tar.gz";
-	    }
-	}
-	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
-	    if ($symlink eq 'symlink') {
-		$msg .= " and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it";
-	    } elsif ($symlink eq 'rename') {
-		$msg .= " and renamed it as ${pkg}_${newversion}.orig.tar.bz2";
-	    }
-	}
-	dehs_msg($msg);
-    } else {
-	print "$pkg: Successfully downloaded updated package $newfile_base\n";
-	if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
-	    if ($symlink eq 'symlink') {
-		print "    and symlinked ${pkg}_${newversion}.orig.tar.gz to it\n";
-	    } elsif ($symlink eq 'rename') {
-		print "    and renamed it as ${pkg}_${newversion}.orig.tar.gz\n";
-	    }
-	}
-	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
-	    if ($symlink eq 'symlink') {
-		print "    and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it\n";
-	    } elsif ($symlink eq 'rename') {
-		print "    and renamed it as ${pkg}_${newversion}.orig.tar.bz2\n";
+		print "    and renamed it as $renamed_base\n";
 	    }
 	}
+	last;
     }
 
     # Do whatever the user wishes to do
     if ($action) {
 	my $usefile = "$destdir/$newfile_base";
 	my @cmd = shellwords($action);
-	if ($symlink =~ /^(symlink|rename)$/
-	    and $newfile_base =~ /\.(tar\.gz|tgz)$/) {
-	    $usefile = "$destdir/${pkg}_${newversion}.orig.tar.gz";
-	}
-	elsif ($symlink =~ /^(symlink|rename)$/
-	    and $newfile_base =~ /\.(tar\.bz2|tbz2)$/) {
-	    $usefile = "$destdir/${pkg}_${newversion}.orig.tar.bz2";
+	if ($symlink =~ /^(symlink|rename)$/ && $renamed_base) {
+	    $usefile = "$destdir/$renamed_base";
 	}
 
 	# Any symlink requests are already handled by uscan
@@ -1494,7 +1519,10 @@ sub recursive_regex_dir ($$$) {
 
     $base =~ m%^(\w+://[^/]+)/(.*)$%;
     my $site = $1;
-    my @dirs = split /(\/)/, $2;
+    my @dirs = ();
+    if (defined $2) {
+	@dirs = split /(\/)/, $2;
+    }
     my $dir = '/';
 
     foreach my $dirpattern (@dirs) {
@@ -1555,7 +1583,7 @@ sub newest_dir ($$$$$) {
 	while ($content =~ m/<\s*a\s+[^>]*href\s*=\s*([\"\'])(.*?)\1/gi) {
 	    my $href = $2;
 	    if ($href =~ m&^$dirpattern/?$&) {
-		my $mangled_version = join(".", $href =~ m&^$dirpattern/?$&);
+		my $mangled_version = join(".", map { $_ || '' } $href =~ m&^$dirpattern/?$&);
 		push @hrefs, [$mangled_version, $href];
 	    }
 	}
@@ -1610,7 +1638,7 @@ sub newest_dir ($$$$$) {
 	# We separate out HTMLised listings from standard listings, so
 	# that we can target our search correctly
 	if ($content =~ /<\s*a\s+[^>]*href/i) {
-	    while ($content =~ 
+	    while ($content =~
 		m/(?:<\s*a\s+[^>]*href\s*=\s*\")((?-i)$pattern)\"/gi) {
 		my $dir = $1;
 		my $mangled_version = join(".", $dir =~ m/^$pattern$/);
@@ -1619,12 +1647,14 @@ sub newest_dir ($$$$$) {
 	} else {
 	    # they all look like:
 	    # info info ... info filename [ -> linkname]
-	    while ($content =~ m/($pattern)(\s+->\s+\S+)?$/mg) {
-		my $dir = $1;
-		my $mangled_version = join(".", $dir =~ m/^$pattern$/);
-		push @dirs, [$mangled_version, $dir];
+	    foreach my $ln (split(/\n/, $content)) {
+		if ($ln =~ m/($pattern)(\s+->\s+\S+)?$/) {
+		    my $dir = $1;
+		    my $mangled_version = join(".", $dir =~ m/^$pattern$/);
+		    push @dirs, [$mangled_version, $dir];
+		}
 	    }
-	}	    
+	}
 	if (@dirs) {
 	    if ($debug) {
 		print STDERR "-- Found the following matching dirs:\n";
@@ -1696,7 +1726,7 @@ sub process_watchfile ($$$$)
 	} elsif ($download == 0 and ! $dehs) {
 	    $pkg_report_header = "Processing watchfile line for package $package...\n";
 	}
-	    
+
 	$status +=
 	    process_watchline($_, $watch_version, $dir, $package, $version,
 			      $watchfile);
@@ -1746,7 +1776,7 @@ sub dehs_output ()
 
     for my $tag (qw(package debian-uversion debian-mangled-uversion
 		    upstream-version upstream-url
-		    status messages warnings errors)) {
+		    status target messages warnings errors)) {
 	if (exists $dehs_tags{$tag}) {
 	    if (ref $dehs_tags{$tag} eq "ARRAY") {
 		foreach my $entry (@{$dehs_tags{$tag}}) {
@@ -1862,7 +1892,7 @@ sub safe_replace($$) {
     if ($op eq 'tr' or $op eq 'y') {
 	$safeflags =~ tr/cds//cd;
 	return 0 if $safeflags ne $flags;
-	
+
 	$regexp =~ s/\\(.)/$1/g;
 	$replacement =~ s/\\(.)/$1/g;
 
