@@ -17,8 +17,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 
 # Command line syntax is one of:
@@ -108,8 +107,8 @@ DEFAULT_UUPDATE_SYMLINK_ORIG=yes
 VARS="UUPDATE_ROOTCMD UUPDATE_PRISTINE UUPDATE_SYMLINK_ORIG"
 SUFFIX="1"
 
-if which lsb_release >/dev/null 2>&1; then
-    case "$(lsb_release --short --id 2>/dev/null)" in
+if which dpkg-vendor >/dev/null 2>&1; then
+    case "$(dpkg-vendor --query Vendor 2>/dev/null)" in
 	"Ubuntu")
 	    SUFFIX="0ubuntu1"
 	    ;;
@@ -174,8 +173,7 @@ TEMP=$(getopt -s bash -o v:p:r:ubs \
 	--long pristine,no-pristine,nopristine \
 	--long symlink,no-symlink,nosymlink \
 	--long no-conf,noconf \
-	--long help,version -n "$PROGNAME" -- "$@")
-if [ $? != 0 ] ; then exit 1 ; fi
+	--long help,version -n "$PROGNAME" -- "$@") || (usage >&2; exit 1)
 
 eval set -- $TEMP
 
@@ -266,7 +264,7 @@ if [ "$PATCH" ]; then
 	    case "$PATCH" in
 		*.gz)  CATPATCH="zcat $PATCH"; X=${X%.gz};;
 		*.bz2) CATPATCH="bzcat $PATCH"; X=${X%.bz2};;
-		*.lzma) CATPATCH="lzcat $PATCH"; X=${X%.lzma};;
+		*.lzma) CATPATCH="xz -F lzma -dc $PATCH"; X=${X%.lzma};;
 		*.xz) CATPATCH="xzcat $PATCH"; X=${X%.xz};;
 		*)     CATPATCH="cat $PATCH";;
 	    esac
@@ -295,9 +293,9 @@ if [ "$PATCH" ]; then
 		    ;;
 		*.lzma)
 		    if [ -r "$OPWD/$PATCH" ]; then
-			CATPATCH="lzcat $OPWD/$PATCH"
+			CATPATCH="xz -F lzma -dc $OPWD/$PATCH"
 		    else
-			CATPATCH="lzcat ../$PATCH"
+			CATPATCH="xz -F lzma -dc ../$PATCH"
 		    fi
 		    X=${X%.lzma}
 		    ;;
@@ -309,7 +307,8 @@ if [ "$PATCH" ]; then
 		    fi
 		    X=${X%.xz}
 		    ;;
-		*)    if [ -r "$OPWD/$PATCH" ]; then
+		*)
+		    if [ -r "$OPWD/$PATCH" ]; then
 			CATPATCH="cat $OPWD/$PATCH"
 		    else
 			CATPATCH="cat ../$PATCH"
@@ -500,7 +499,7 @@ else
 		echo "$PROGNAME: cannot read archive file $ARCHIVE!  Aborting." >&2
 		exit 1
 	    fi
-	    
+
 	    ;;
     esac
 
@@ -686,6 +685,9 @@ else
 	# Otherwise, we put them into a new directory
 	mkdir $PACKAGE-$SNEW_VERSION
 	mv $TEMP_DIR/* $PACKAGE-$SNEW_VERSION
+	if ls $TEMP_DIR/.[!.]* >/dev/null 2>&1 ; then
+	    mv $TEMP_DIR/.[!.]* $PACKAGE-$SNEW_VERSION
+	fi
     fi
     rm -rf $TEMP_DIR
     cp -a $PACKAGE-$SNEW_VERSION $PACKAGE-$SNEW_VERSION.orig
@@ -702,7 +704,7 @@ else
     elif [ -r "../${PACKAGE}_$SVERSION.diff.lzma" ]; then
 	DIFF="../${PACKAGE}_$SVERSION.diff.lzma"
 	DIFFTYPE=diff
-	DIFFCAT=lzcat
+	DIFFCAT="xz -F lzma -dc"
     elif [ -r "../${PACKAGE}_$SVERSION.diff.xz" ]; then
 	DIFF="../${PACKAGE}_$SVERSION.diff.xz"
 	DIFFTYPE=diff
@@ -754,7 +756,7 @@ else
 		    echo "Please check that the diff is applied correctly." >&2
 		    echo "(This program will use the pristine upstream version and save the old .diff.gz" >&2
 		    echo "version as $file.debdiff .)" >&2
-		    
+
 		    if [ -e "$file.upstream" -o -e "$file.debdiff" ]; then
 			FILEEXISTERR=1
 		    fi
@@ -777,6 +779,14 @@ else
 	    done
 	fi
 
+	# Remove all existing symlinks before applying the patch.  We'll
+	# restore them afterwards, but this avoids patch following symlinks,
+	# which may point outside of the source tree
+	declare -a LINKS
+	while IFS= read -d '' -r link; do
+	    LINKS+=("$link")
+	done < <(find -type l -printf '%l\0%p\0' -delete)
+
 	if $DIFFCAT $DIFF | patch -sNp1 ; then
 	    echo "Success!  The diffs from version $VERSION worked fine."
 	else
@@ -788,6 +798,16 @@ else
 	    STATUS=1
 	fi
 
+	# Reinstate symlinks, warning if the
+	for (( i=0; $i < ${#LINKS[@]}; i=$(($i+2)) )); do
+	    target="${LINKS[$i]}"
+	    link="${LINKS[$(($i+1))]}"
+	    if ! ln -s -T "$target" "$link"; then
+		echo "$PROGNAME: warning: Unable to restore the '$link' -> '$target' symlink." >&2
+		STATUS=1
+	    fi
+	done
+
 	for file in "${MOVEDFILES[@]}"; do
 	    if [ -e "$file.upstream" ]; then
 		mv $file $file.debdiff
@@ -809,7 +829,7 @@ else
 	fi
 	if $DIFFUNPACK $DIFF; then
 	    echo "Unpacking the debian/ directory from version $VERSION worked fine."
-	else 
+	else
 	    echo "$PROGNAME: failed to unpack the debian/ directory from version $VERSION!" >&2
 	    exit 1
 	fi
